@@ -1,15 +1,17 @@
 # Copyright 2009-2011 by Peter Cock.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 """Dictionary like indexing of sequence files (PRIVATE).
 
 You are not expected to access this module, or any of its code, directly. This
-is all handled internally by the Bio.SeqIO.index(...) function which is the
-public interface for this functionality.
+is all handled internally by the Bio.SeqIO.index(...) and index_db(...)
+functions which are the public interface for this functionality.
 
 The basic idea is that we scan over a sequence file, looking for new record
-markers. We then try and extract the string that Bio.SeqIO.parse/read would
+markers. We then try to extract the string that Bio.SeqIO.parse/read would
 use as the record id, ideally without actually parsing the full record. We
 then use a subclassed Python dictionary to record the file offset for the
 record start against the record id.
@@ -19,8 +21,9 @@ record may not trigger an exception until it is accessed. This is by design.
 
 This means our dictionary like objects have in memory ALL the keys (all the
 record identifiers), which shouldn't be a problem even with second generation
-sequencing. If this is an issue later on, storing the keys and offsets in a
-temp lookup file might be one idea (e.g. using SQLite or an OBDA style index).
+sequencing. If memory is an issue, the index_db(...) interface stores the
+keys and offsets in an SQLite database - which can be re-used to avoid
+re-indexing the file for use another time.
 """
 
 from __future__ import print_function
@@ -37,6 +40,7 @@ from Bio.File import _IndexedSeqFileProxy, _open_for_random_access
 
 class SeqFileRandomAccess(_IndexedSeqFileProxy):
     def __init__(self, filename, format, alphabet):
+        """Initialize the class."""
         self._handle = _open_for_random_access(filename)
         self._alphabet = alphabet
         self._format = format
@@ -60,7 +64,7 @@ class SeqFileRandomAccess(_IndexedSeqFileProxy):
         self._parse = _parse
 
     def get(self, offset):
-        """Returns SeqRecord."""
+        """Return SeqRecord."""
         # Should be overridden for binary file formats etc:
         return self._parse(StringIO(_bytes_to_string(self.get_raw(offset))))
 
@@ -76,6 +80,7 @@ class SffRandomAccess(SeqFileRandomAccess):
     """Random access to a Standard Flowgram Format (SFF) file."""
 
     def __init__(self, filename, format, alphabet):
+        """Initialize the class."""
         SeqFileRandomAccess.__init__(self, filename, format, alphabet)
         header_length, index_offset, index_length, number_of_reads, \
             self._flows_per_read, self._flow_chars, self._key_sequence \
@@ -100,9 +105,9 @@ class SffRandomAccess(SeqFileRandomAccess):
                     max_offset = max(max_offset, offset)
                     yield name, offset, 0
                     count += 1
-                assert count == number_of_reads, \
-                    "Indexed %i records, expected %i" \
-                    % (count, number_of_reads)
+                if count != number_of_reads:
+                    raise ValueError("Indexed %i records, expected %i"
+                                     % (count, number_of_reads))
                 # If that worked, call _check_eof ...
             except ValueError as err:
                 import warnings
@@ -129,8 +134,9 @@ class SffRandomAccess(SeqFileRandomAccess):
         for name, offset in SeqIO.SffIO._sff_do_slow_index(handle):
             yield name, offset, 0
             count += 1
-        assert count == number_of_reads, \
-            "Indexed %i records, expected %i" % (count, number_of_reads)
+        if count != number_of_reads:
+            raise ValueError("Indexed %i records, expected %i"
+                             % (count, number_of_reads))
         SeqIO.SffIO._check_eof(handle, index_offset, index_length)
 
     def get(self, offset):
@@ -167,6 +173,7 @@ class SffTrimedRandomAccess(SffRandomAccess):
 
 class SequentialSeqFileRandomAccess(SeqFileRandomAccess):
     def __init__(self, filename, format, alphabet):
+        """Initialize the class."""
         SeqFileRandomAccess.__init__(self, filename, format, alphabet)
         marker = {"ace": b"CO ",
                   "embl": b"ID ",
@@ -184,7 +191,7 @@ class SequentialSeqFileRandomAccess(SeqFileRandomAccess):
         self._marker_re = re.compile(b"^" + marker)
 
     def __iter__(self):
-        """Returns (id, offset, length) tuples."""
+        """Return (id, offset, length) tuples."""
         marker_offset = len(self._marker)
         marker_re = self._marker_re
         handle = self._handle
@@ -252,25 +259,39 @@ class GenBankRandomAccess(SequentialSeqFileRandomAccess):
         while marker_re.match(line):
             # We cannot assume the record.id is the first word after LOCUS,
             # normally the first entry on the VERSION or ACCESSION line is used.
-            key = None
+            # However if both missing, GenBank parser falls back on LOCUS entry.
+            try:
+                key = line[5:].split(None, 1)[0]
+            except ValueError:
+                # Warning?
+                # No content in LOCUS line
+                key = None
             length = len(line)
             while True:
                 end_offset = handle.tell()
                 line = handle.readline()
                 if marker_re.match(line) or not line:
                     if not key:
-                        raise ValueError(
-                            "Did not find ACCESSION/VERSION lines")
+                        raise ValueError("Did not find usable ACCESSION/VERSION/LOCUS lines")
                     yield _bytes_to_string(key), start_offset, length
                     start_offset = end_offset
                     break
                 elif line.startswith(accession_marker):
-                    key = line.rstrip().split()[1]
+                    try:
+                        key = line.rstrip().split()[1]
+                    except IndexError:
+                        # No content in ACCESSION line
+                        pass
                 elif line.startswith(version_marker):
-                    version_id = line.rstrip().split()[1]
-                    if version_id.count(b".") == 1 and version_id.split(b".")[1].isdigit():
-                        # This should mimic the GenBank parser...
-                        key = version_id
+                    try:
+                        version_id = line.rstrip().split()[1]
+                        if version_id.count(b".") == 1 and version_id.split(b".")[1].isdigit():
+                            # This should mimic the GenBank parser...
+                            key = version_id
+                    except IndexError:
+                        # No content in VERSION line
+                        pass
+
                 length += len(line)
         assert not line, repr(line)
 
@@ -456,6 +477,7 @@ class IntelliGeneticsRandomAccess(SeqFileRandomAccess):
     """Random access to a IntelliGenetics file."""
 
     def __init__(self, filename, format, alphabet):
+        """Initialize the class."""
         SeqFileRandomAccess.__init__(self, filename, format, alphabet)
         self._marker_re = re.compile(b"^;")
 
